/*                   Listing 4
    Windows vertical button member functions for C++
          by Norman L. Hills & Davin S. Hills
    Copyright (c) 1992 Hills Object Technology, Inc.
                 All Rights Reserved
*/

#ifndef SubclassWindow
#define SubclassWindow(hwnd, lpfn)  \
                ((WNDPROC)SetWindowLong((hwnd),  \
                GWL_WNDPROC, (LPARAM)(WNDPROC)(lpfn)))
#endif

/* Instantiate a VButton object
*/
CCVPTR cc_VertButton(HWND hwnd, HINSTANCE hInst,
                     int winID, LPSTR lptext,
                     int X = -1, int Y = -1,
                     int width = 0, int height = 0) {
    CCVPTR  vertptr;
    vertptr = new VButton(hwnd, hInst, winID, lptext,
                          X, Y, width, height);
    vertptr->Start();
    return vertptr;
}
/* Windows Function that can access a VButton class
   member function via the Property List
*/
LRESULT CALLBACK _export cc_VertProc(HWND shwnd,
                                     UINT msg,
                                     WPARAM wP,
                                     LPARAM lP)   {
    CCVPTR  vptr;
    vptr = (CCVPTR)GetCObjptr(shwnd);
    if (vptr)
        return vptr->cc_DoVertProc(shwnd,msg,wP,lP);
    return DefWindowProc(shwnd,msg,wP,lP);
}

int VButton::SetupButton(LPSTR lpsettext,
                                  BOOL setsize)   {
          lpText = (LPSTR)&tstr;
          lstrcpy(lpText,lpsettext);
          ilen = lstrlen(lpText);
          ampindex = NULL;
          for (lpText; *lpText; lpText++)  {
               if (*lpText == 38)  {          //  &
                   ampindex = lpText;
                   winstr[1] = *++lpText;
                   break;
               }
          }
          lpText = (LPSTR)&tstr;
          if (setsize)   {
              wd = charwd + 16;
              ht = (charht * ilen) + 16;
              if (ampindex)
                  ht -= charht;
              hfoc = 6;
              vfoc = 0;
           }
           return (ilen);
}
/* Constructor
*/
VButton::VButton(HWND hwnd, HINSTANCE hInstance,
                 int winID, LPSTR lptext,
                 int X, int Y,
                 int width, int height)  {
                              DWORD  dsize;
                              HDC  aDC;
                              int  xunits, yunits;
          curwnd = hwnd;
          hInst = hInstance;
          aDC = GetDC(hwnd);
          dsize = GetTextExtent(aDC,"W",1);
          ReleaseDC(hwnd,aDC);
          charwd = LOWORD(dsize);
          charht = HIWORD(dsize);
          winstr[0] = 38;
          winstr[1] = '\0';
          winstr[2] = '\0';
          tstr[0] = '\0';
          xunits = LOWORD(GetDialogBaseUnits());
          yunits = HIWORD(GetDialogBaseUnits());

          if (X == -1)  {
              hwnde = GetDlgItem(hwnd,winID);
              GetWindowRect(hwnde,(LPRECT)&wrect);
              ScreenToClient(hwnd,
                             (LPPOINT)&wrect.left);
              ScreenToClient(hwnd,
                             (LPPOINT)&wrect.right);
              width = wrect.right - wrect.left;
              height = wrect.bottom - wrect.top;
              // size is adjusted, therefore...
              xunits = 4;    yunits = 8;
              WinX = wrect.left;
              WinY = wrect.top;
          }
          else  {
              WinX = (X * xunits) / 4;
              WinY = (Y * yunits) / 8;
          }
          olen = SetupButton(lptext,TRUE);
          width = (width * xunits) / 4;
          if ((width > wd) || (X == -1))  {
              wd = width;
              hfoc = ((width - charwd) / 2) - 3;
          }
          height = (height * yunits) / 8;
          if ((height > ht) || (X == -1))
              ht = height;
        SetRect((LPRECT)&wrect,WinX+hfoc,WinY+vfoc+6,
                        WinX+wd-hfoc,WinY+ht-vfoc-6);
          state = 0;

          if (X != -1)
//--------------
    hwnde = CreateWindow("button", (LPSTR)&winstr,
                         (WS_CHILD | WS_VISIBLE |
                          WS_TABSTOP | BS_PUSHBUTTON),
                          WinX,WinY,wd,ht,hwnd,(HMENU)winID,
                          hInstance,NULL);
//--------------
          else   // activate possible Alt-key
             SetWindowText(hwnde,(LPSTR)&winstr);
}

/* Start the VButton now that "this" is available
*/
void VButton::Start(void)  {            RECT trect;
    if (!lpfnOldVertProc)
        MakeProcInstance((FARPROC)cc_VertProc,hInst);
    SetCObjptr(hwnde,(CCVPTR)this);
   lpfnOldVertProc = SubclassWindow(hwnde,cc_VertProc);
    GetWindowRect(curwnd,(LPRECT)&trect);
    InvalidateRect(curwnd,(LPRECT)&trect,TRUE);
}

void VButton::DoVText(HDC vhdc,LPRECT irect)  {
                                    HBRUSH  hbrush;
                                    HPEN    hpen;
                                    RECT    vrect;
                                    LPSTR   lpvtext;
                                    COLORREF otc, obc;
   CopyRect(&vrect,irect);
   hbrush = CreateSolidBrush
                    (GetSysColor(COLOR_BTNFACE));
   hpen = CreatePen(PS_SOLID,1,
                    GetSysColor(COLOR_BTNFACE));
   SelectObject(vhdc,hbrush);
   SelectObject(vhdc,hpen);
   Rectangle(vhdc,vrect.left,vrect.top,vrect.right,
                  vrect.bottom);
   SelectObject(vhdc,GetStockObject(WHITE_BRUSH));
   SelectObject(vhdc,GetStockObject(BLACK_PEN));
   DeleteObject(hbrush);
   DeleteObject(hpen);
   obc = SetBkColor(vhdc,GetSysColor(COLOR_BTNFACE));
   otc = SetTextColor(vhdc,
                      GetSysColor(COLOR_BTNTEXT));
   vrect.top += vfoc + 1;
   vrect.bottom = vrect.top + charht;
   for (lpvtext = lpText; *lpvtext; lpvtext++)  {
      if (lpvtext != ampindex)
         ::DrawText(vhdc,lpvtext,1,&vrect,
           (DT_CENTER | DT_VCENTER | DT_SINGLELINE));
      else  {
         ::DrawText(vhdc,lpvtext,2,&vrect,
           (DT_CENTER | DT_VCENTER | DT_SINGLELINE));
         lpvtext++;   //  bump an extra
      }
      vrect.top = vrect.bottom;
      vrect.bottom += charht;
   }
   SetBkColor(vhdc,obc);
   SetTextColor(vhdc,otc);
}
/* The subclassed Windows "WNDPROC"
*/
LRESULT VButton::cc_DoVertProc(HWND dhwnd, UINT msg,
                          WPARAM wP, LPARAM lP)    {
                                        HDC  shdc;
                                       RECT  prect;
                                    LRESULT  retval;

   retval = CallWindowProc((WNDPROC)lpfnOldVertProc,
                           dhwnd,msg,wP,lP);
   if ((msg == WM_SETFOCUS) || (msg == WM_PAINT))  {

       shdc = GetDC(GetParent(dhwnd));
       CopyRect((LPRECT)&prect,(LPRECT)&wrect);
       if (state)
           OffsetRect((LPRECT)&prect,1,2);
       DoVText(shdc,(LPRECT)&prect);
       if (hwnde == ::GetFocus())
           DrawFocusRect(shdc,(LPRECT)&prect);
       ReleaseDC(GetParent(dhwnd),shdc);
       return TRUE;
   }
   if (msg == BM_SETSTATE)  {

       if ((!wP && !state) || (wP && state))
            return FALSE;    // Stops the flicker!
       state = wP;
       SendMessage(dhwnd,WM_PAINT,0,0L);
       return TRUE;
   }
   return retval;  //  MUST save from call & use
}
/* Auxiliary class functions
*/
void VButton::SetLabel(LPSTR lpnewtext,BOOL resize) {
                                   RECT  trect;
    if ((lstrlen(lpnewtext) <= olen) || (resize))  {
        SetupButton(lpnewtext, resize);
        if (resize)  {
            MoveWindow(hwnde,WinX,WinY,wd,ht,TRUE);
            olen = ilen;
        }
        SetWindowText(hwnde,(LPSTR)&winstr);
        SetRect((LPRECT)&wrect,WinX+hfoc,WinY+vfoc+6,
                WinX+wd-hfoc, WinY+ht-vfoc-6);
        GetWindowRect(curwnd,(LPRECT)&trect);
        InvalidateRect(curwnd,(LPRECT)&trect,TRUE);
    }
}
LPSTR VButton::GetLabel(void)   {
                          return (LPSTR)lpText;  }

void VButton::Focus(void)  {  ::SetFocus(hwnde);  }

     VButton::~VButton()   {  RemoveCObjptr(hwnde);  }

WNDPROC  VButton::lpfnOldVertProc;
